@@ -1,4 +1,4 @@
-{-# LANGUAGE CPP             #-}
+{-# LANGUAGE CPP #-}
 {-# LANGUAGE TemplateHaskell #-}
 
 module Main (main) where
@@ -16,10 +16,13 @@ import Test.Hspec
 
 main :: IO ()
 main = hspec . around withSandbox $ do
+
+{- ORMOLU_DISABLE -}
+
 #ifndef mingw32_HOST_OS
   beforeWith populatedDir $ do
-    -- NOTE These tests shall fail on Windows as unix-compat does not
-    -- implement createSymbolicLink for windows.
+    -- NOTE These tests fail on Windows as unix-compat does not implement
+    -- createSymbolicLink for windows.
     describe "listDir"          listDirSpec
     describe "listDirRel"       listDirRelSpec
     describe "listDirRecur"     listDirRecurSpec
@@ -29,13 +32,14 @@ main = hspec . around withSandbox $ do
     describe "copyDirRecur"     copyDirRecurSpec
     describe "copyDirRecur'"    copyDirRecur'Spec
     describe "findFile"         findFileSpec
+    describe "removeDirLink"    removeDirLinkSpec
   beforeWith populatedCyclicDir $
     describe "listDirRecur Cyclic" listDirRecurCyclicSpec
 #endif
-  describe "getCurrentDir"    getCurrentDirSpec
-  describe "setCurrentDir"    setCurrentDirSpec
-  describe "withCurrentDir"   withCurrentDirSpec
-  describe "walkDirRel"       walkDirRelSpec
+  describe "getCurrentDir" getCurrentDirSpec
+  describe "setCurrentDir" setCurrentDirSpec
+  describe "withCurrentDir" withCurrentDirSpec
+  describe "walkDirRel" walkDirRelSpec
 #ifndef mingw32_HOST_OS
   -- NOTE We can't quite test this on Windows as well, because the
   -- environmental variables HOME and TMPDIR do not exist there.
@@ -46,6 +50,8 @@ main = hspec . around withSandbox $ do
   describe "getXdgDir Cache"  getXdgCacheDirSpec
 #endif
 
+{- ORMOLU_ENABLE -}
+
 listDirSpec :: SpecWith (Path Abs Dir)
 listDirSpec = it "lists directory" $ \dir ->
   getDirStructure listDir dir `shouldReturn` populatedDirTop
@@ -65,70 +71,77 @@ listDirRecurRelSpec = it "lists directory recursively" $ \dir ->
 listDirRecurWithSpec :: SpecWith (Path Abs Dir)
 listDirRecurWithSpec =
   it "lists directory recursively using predicates" $ \dir ->
-      getDirStructure (listDirRecurWith
-                        (return . ($(mkRelDir "c") /=) . dirname)
-                        (return . ($(mkRelFile "two.txt") /=) . filename)) dir
-        `shouldReturn` populatedDirRecurWith
+    getDirStructure
+      ( listDirRecurWith
+          (return . ($(mkRelDir "c") /=) . dirname)
+          (return . ($(mkRelFile "two.txt") /=) . filename)
+      )
+      dir
+      `shouldReturn` populatedDirRecurWith
 
-listDirRecurWith
-  :: (Path Abs Dir ->  IO Bool)           -- ^ Dir match predicate
-  -> (Path Abs File -> IO Bool)           -- ^ File match predicate
-  -> Path Abs Dir                         -- ^ Top dir to traverse
-  -> IO ([Path Abs Dir], [Path Abs File]) -- ^ Matched subdirs and files
+listDirRecurWith ::
+  -- | Dir match predicate
+  (Path Abs Dir -> IO Bool) ->
+  -- | File match predicate
+  (Path Abs File -> IO Bool) ->
+  -- | Top dir to traverse
+  Path Abs Dir ->
+  -- | Matched subdirs and files
+  IO ([Path Abs Dir], [Path Abs File])
 listDirRecurWith dirPred filePred =
   walkDirAccum Nothing $ \_ d f -> do
-    d' <- filterM dirPred  d
+    d' <- filterM dirPred d
     f' <- filterM filePred f
     return (d', f')
 
--- | walkDir with a Finish handler may have unpredictable output depending on
--- the order of traversal. The only guarantee is that we will finish only after
--- we find the directory "c". Though if we test only for the presence of "c" we
--- are not really testing if we indeed cut the traversal short.
-
+-- | 'walkDir' with a 'WalkFinish' handler may have unpredictable output
+-- depending on the order of traversal. The only guarantee is that we will
+-- finish only after we find the directory "c". Though if we test only for
+-- the presence of "c" we are not really testing if we indeed cut the
+-- traversal short.
 walkDirFinishSpec :: SpecWith (Path Abs Dir)
 walkDirFinishSpec =
   it "Finishes only after finding what it is looking for" $ \dir -> do
     (d, _) <- getDirStructure (walkDirAccum (Just dHandler) writer) dir
     map dirname d `shouldContain` [$(mkRelDir "c")]
-    where dHandler p _ _
-            | dirname p == $(mkRelDir "c") = return WalkFinish
-            | otherwise = return (WalkExclude [])
-
-          writer _ d f = return (d, f)
+  where
+    dHandler p _ _
+      | dirname p == $(mkRelDir "c") = return WalkFinish
+      | otherwise = return (WalkExclude [])
+    writer _ d f = return (d, f)
 
 copyDirRecurSpec :: SpecWith (Path Abs Dir)
 copyDirRecurSpec = do
   context "when source directory is editable" $
     it "copies directory" $ \src -> do
-    let dest = parent src </> $(mkRelDir "copied-dir")
-    copyDirRecur src dest
-    old <- getDirStructure listDirRecur src
-    new <- getDirStructure listDirRecur dest
-    old `shouldBe` new
+      let dest = parent src </> $(mkRelDir "copied-dir")
+      copyDirRecur src dest
+      old <- getDirStructure listDirRecur src
+      new <- getDirStructure listDirRecur dest
+      old `shouldBe` new
   context "when source directory is read-only" $
     it "copies directory just as well (preserving permissions)" $ \src -> do
-    let dest = parent src </> $(mkRelDir "copied-dir")
-    srcPermissions <- setOwnerWritable False <$> getPermissions src
-    setPermissions src srcPermissions
-    copyDirRecur src dest
-    old <- getDirStructure listDirRecur src
-    new <- getDirStructure listDirRecur dest
-    old `shouldBe` new
-    getPermissions dest `shouldReturn` srcPermissions
+      let dest = parent src </> $(mkRelDir "copied-dir")
+      srcPermissions <- setOwnerWritable False <$> getPermissions src
+      setPermissions src srcPermissions
+      copyDirRecur src dest
+      old <- getDirStructure listDirRecur src
+      new <- getDirStructure listDirRecur dest
+      old `shouldBe` new
+      getPermissions dest `shouldReturn` srcPermissions
 
 copyDirRecur'Spec :: SpecWith (Path Abs Dir)
 copyDirRecur'Spec =
   context "when source directory is read-only" $
     it "copies directory but now it's editable" $ \src -> do
-    let dest = parent src </> $(mkRelDir "copied-dir")
-    srcPermissions <- setOwnerWritable False <$> getPermissions src
-    setPermissions src srcPermissions
-    copyDirRecur' src dest
-    old <- getDirStructure listDirRecur src
-    new <- getDirStructure listDirRecur dest
-    old `shouldBe` new
-    getPermissions dest `shouldReturn` srcPermissions { writable = True }
+      let dest = parent src </> $(mkRelDir "copied-dir")
+      srcPermissions <- setOwnerWritable False <$> getPermissions src
+      setPermissions src srcPermissions
+      copyDirRecur' src dest
+      old <- getDirStructure listDirRecur src
+      new <- getDirStructure listDirRecur dest
+      old `shouldBe` new
+      getPermissions dest `shouldReturn` srcPermissions {writable = True}
 
 findFileSpec :: SpecWith (Path Abs Dir)
 findFileSpec = it "finds a file lazily" $ \dir -> do
@@ -136,16 +149,28 @@ findFileSpec = it "finds a file lazily" $ \dir -> do
   found <- findFile (dir : undefined) relFile
   found `shouldBe` Just (dir </> relFile)
 
+removeDirLinkSpec :: SpecWith (Path Abs Dir)
+removeDirLinkSpec = it "remove dir link" $ \dir -> do
+  let target = dir </> $(mkRelDir "a")
+      link = dir </> $(mkRelDir "link-a")
+  createDirLink target link
+  removeDirLink link
+  exists <- doesDirExist link
+  exists `shouldBe` False
+
 listDirRecurCyclicSpec :: SpecWith (Path Abs Dir)
 listDirRecurCyclicSpec =
   it "lists directory trees having traversal cycles" $ \dir ->
     getDirStructure listDirRecurCyclic dir
-        `shouldReturn` populatedCyclicDirStructure
+      `shouldReturn` populatedCyclicDirStructure
 
--- Follows symbolic links
-listDirRecurCyclic :: (MonadIO m, MonadThrow m)
-  => Path b Dir                          -- ^ Directory to list
-  -> m ([Path Abs Dir], [Path Abs File]) -- ^ Sub-directories and files
+-- | Follows symbolic links.
+listDirRecurCyclic ::
+  (MonadIO m, MonadThrow m) =>
+  -- | Directory to list
+  Path b Dir ->
+  -- | Sub-directories and files
+  m ([Path Abs Dir], [Path Abs File])
 listDirRecurCyclic = walkDirAccum Nothing (\_ d f -> return (d, f))
 
 getCurrentDirSpec :: SpecWith (Path Abs Dir)
@@ -156,7 +181,7 @@ setCurrentDirSpec :: SpecWith (Path Abs Dir)
 setCurrentDirSpec = it "sets current dir" $ \dir -> do
   wdir <- getCurrentDir
   setCurrentDir dir
-  new  <- getCurrentDir
+  new <- getCurrentDir
   setCurrentDir wdir
   new `shouldBe` dir
 
@@ -181,16 +206,18 @@ getHomeDirSpec =
     bracket (getEnv evar) (setEnv evar) $ \_ -> do
       setEnv evar (toFilePath dir)
       getHomeDir `shouldReturn` dir
-  where evar = "HOME"
+  where
+    evar = "HOME"
 
 getTempDirSpec :: SpecWith (Path Abs Dir)
 getTempDirSpec =
   it "temp dir is influenced by environment variable TMPDIR" $ \dir ->
-    flip finally (unsetEnv evar) $  do
+    flip finally (unsetEnv evar) $ do
       setEnv evar (toFilePath dir)
       getTempDir `shouldReturn` dir
       unsetEnv evar
-  where evar = "TMPDIR"
+  where
+    evar = "TMPDIR"
 
 getXdgDataDirSpec :: SpecWith (Path Abs Dir)
 getXdgDataDirSpec =
@@ -200,8 +227,9 @@ getXdgDataDirSpec =
       getXdgDir XdgData (Just name) `shouldReturn` (dir </> name)
       getXdgDir XdgData Nothing `shouldReturn` dir
       unsetEnv evar
-  where evar = "XDG_DATA_HOME"
-        name = $(mkRelDir "test")
+  where
+    evar = "XDG_DATA_HOME"
+    name = $(mkRelDir "test")
 
 getXdgConfigDirSpec :: SpecWith (Path Abs Dir)
 getXdgConfigDirSpec =
@@ -211,8 +239,9 @@ getXdgConfigDirSpec =
       getXdgDir XdgConfig (Just name) `shouldReturn` (dir </> name)
       getXdgDir XdgConfig Nothing `shouldReturn` dir
       unsetEnv evar
-  where evar = "XDG_CONFIG_HOME"
-        name = $(mkRelDir "test")
+  where
+    evar = "XDG_CONFIG_HOME"
+    name = $(mkRelDir "test")
 
 getXdgCacheDirSpec :: SpecWith (Path Abs Dir)
 getXdgCacheDirSpec =
@@ -222,17 +251,16 @@ getXdgCacheDirSpec =
       getXdgDir XdgCache (Just name) `shouldReturn` (dir </> name)
       getXdgDir XdgCache Nothing `shouldReturn` dir
       unsetEnv evar
-  where evar = "XDG_CACHE_HOME"
-        name = $(mkRelDir "test")
+  where
+    evar = "XDG_CACHE_HOME"
+    name = $(mkRelDir "test")
 
 ----------------------------------------------------------------------------
 -- Helpers
 
 -- | Create a sandbox directory to model some situation in it and run some
--- tests. Note that we're using new unique sandbox directory for each test
--- case to avoid contamination and it's unconditionally deleted after test
--- case finishes.
-
+-- tests. Note that we're using a new unique sandbox directory for each test
+-- case and it's unconditionally deleted after test case finishes.
 withSandbox :: ActionWith (Path Abs Dir) -> IO ()
 withSandbox = withSystemTempDir "path-io-sandbox"
 
@@ -240,11 +268,10 @@ withSandbox = withSystemTempDir "path-io-sandbox"
 -- to that directory.
 --
 -- Created objects are described in 'populatedDirStructure'.
-
 populatedDir :: Path Abs Dir -> IO (Path Abs Dir)
 populatedDir root = do
   let (_, files) = populatedDirStructure
-      pdir          = root </> $(mkRelDir "pdir")
+      pdir = root </> $(mkRelDir "pdir")
       withinSandbox = (pdir </>)
   ensureDir pdir
   ensureDir $ withinSandbox $(mkRelDir "b")
@@ -255,44 +282,41 @@ populatedDir root = do
   forM_ files $ (`writeFile` "") . toFilePath . withinSandbox
   return pdir
 
--- | Get inner structure of a directory. Items are sorted, so it's easier to
--- compare results.
-
-getDirStructure
-  :: (Path Abs Dir -> IO ([Path Abs Dir], [Path Abs File]))
-     -- ^ Which function to use for scanning
-  -> Path Abs Dir
-     -- ^ Path to directory to scan
-  -> IO ([Path Rel Dir], [Path Rel File])
+-- | Get the inner structure of a directory. Items are sorted, so it's
+-- easier to compare results.
+getDirStructure ::
+  -- | Which function to use for scanning
+  (Path Abs Dir -> IO ([Path Abs Dir], [Path Abs File])) ->
+  -- | Path to directory to scan
+  Path Abs Dir ->
+  IO ([Path Rel Dir], [Path Rel File])
 getDirStructure f path = do
   (dirs, files) <- f path
-  rdirs  <- sort <$> mapM (makeRelative path) dirs
+  rdirs <- sort <$> mapM (makeRelative path) dirs
   rfiles <- sort <$> mapM (makeRelative path) files
   return (rdirs, rfiles)
 
 -- | A version of 'getDirStructure' that accepts scanning function that
 -- returns relative paths.
-
-getDirStructureRel
-  :: (Path Abs Dir -> IO ([Path Rel Dir], [Path Rel File]))
-     -- ^ Which function to use for scanning
-  -> Path Abs Dir
-     -- ^ Path to directory to scan
-  -> IO ([Path Rel Dir], [Path Rel File])
+getDirStructureRel ::
+  -- | Which function to use for scanning
+  (Path Abs Dir -> IO ([Path Rel Dir], [Path Rel File])) ->
+  -- | Path to directory to scan
+  Path Abs Dir ->
+  IO ([Path Rel Dir], [Path Rel File])
 getDirStructureRel f path = bimap sort sort <$> f path
 
 -- | Structure of directory created by the 'populatedDir' function. Please
 -- keep it sorted.
-
 populatedDirStructure :: ([Path Rel Dir], [Path Rel File])
 populatedDirStructure =
-  ( [ $(mkRelDir "a")
-    , $(mkRelDir "b")
-    , $(mkRelDir "b/c")
-    ]
-  , [ $(mkRelFile "b/c/three.txt")
-    , $(mkRelFile "b/two.txt")
-    , $(mkRelFile "one.txt")
+  ( [ $(mkRelDir "a"),
+      $(mkRelDir "b"),
+      $(mkRelDir "b/c")
+    ],
+    [ $(mkRelFile "b/c/three.txt"),
+      $(mkRelFile "b/two.txt"),
+      $(mkRelFile "one.txt")
     ]
   )
 
@@ -305,29 +329,26 @@ populatedDirStructure =
 --     c\/(d -> a), a\/(b -> c)
 -- 2) Cycle with own ancestor
 --     e\/f\/(g -> e)
-
 populatedCyclicDirStructure :: ([Path Rel Dir], [Path Rel File])
 populatedCyclicDirStructure =
-  ( [
-      $(mkRelDir "a")
-    , $(mkRelDir "a/b")   -- b points to c
-    , $(mkRelDir "a/b/d") -- because b is same as c
-    , $(mkRelDir "c")
-    , $(mkRelDir "c/d")   -- d points to a
-    , $(mkRelDir "c/d/b") -- because d is same as a
-    , $(mkRelDir "e")
-    , $(mkRelDir "e/f")
-    , $(mkRelDir "e/f/g") -- g points to e
-    ]
-  , []
+  ( [ $(mkRelDir "a"),
+      $(mkRelDir "a/b"), -- b points to c
+      $(mkRelDir "a/b/d"), -- because b is same as c
+      $(mkRelDir "c"),
+      $(mkRelDir "c/d"), -- d points to a
+      $(mkRelDir "c/d/b"), -- because d is same as a
+      $(mkRelDir "e"),
+      $(mkRelDir "e/f"),
+      $(mkRelDir "e/f/g") -- g points to e
+    ],
+    []
   )
 
 -- | Created the objects described in 'populatedCyclicDirStructure'.
 -- Return path to that directory.
-
 populatedCyclicDir :: Path Abs Dir -> IO (Path Abs Dir)
 populatedCyclicDir root = do
-  let pdir          = root </> $(mkRelDir "pdir")
+  let pdir = root </> $(mkRelDir "pdir")
       withinSandbox = (pdir </>)
   ensureDir pdir
   ensureDir $ withinSandbox $(mkRelDir "a")
@@ -340,27 +361,25 @@ populatedCyclicDir root = do
 
 -- | Top-level structure of populated directory as it should be scanned by
 -- the 'listDir' function.
-
 populatedDirTop :: ([Path Rel Dir], [Path Rel File])
 populatedDirTop =
-  ( [ $(mkRelDir "a")
-    , $(mkRelDir "b")
-    ]
-  , [ $(mkRelFile "one.txt")
+  ( [ $(mkRelDir "a"),
+      $(mkRelDir "b")
+    ],
+    [ $(mkRelFile "one.txt")
     ]
   )
 
 -- | Structure of populated directory as it should be scanned by
 -- 'listDirRecurWith' function using predicates to filter out dir 'c' and the
 -- file 'two.txt'
-
 populatedDirRecurWith :: ([Path Rel Dir], [Path Rel File])
 populatedDirRecurWith =
-  ( [ $(mkRelDir "a")
-    , $(mkRelDir "b")
-    ]
-  , [ $(mkRelFile "a/c/three.txt") -- via symbolic link
-    , $(mkRelFile "b/c/three.txt")
-    , $(mkRelFile "one.txt")
+  ( [ $(mkRelDir "a"),
+      $(mkRelDir "b")
+    ],
+    [ $(mkRelFile "a/c/three.txt"), -- via symbolic link
+      $(mkRelFile "b/c/three.txt"),
+      $(mkRelFile "one.txt")
     ]
   )
